"""
========================================
  Introduction to Computer Programming
========================================

----------------------------------------------------
  list examples for lab 9
----------------------------------------------------
"""

from random import randrange


def append_first(a):
    """Appends first item on list a onto its end. Assumes a is not empty.

    >>> stuff = ['Cool!', 37, False]
    >>> append_first(stuff)
    >>> stuff
    ['Cool!', 37, False, 'Cool!']
    """
    a.append(a[0])


def decrement_list(a):
    """Subtracts one from every number in list a.

    >>> ns = [44, 66, 55]
    >>> decrement_list(ns)
    >>> ns
    [43, 65, 54]
    """
    for i in range(len(a)):
        a[i] -= 1


def pop_first(a):
    """
    Removes last item from list a and replaces first item with whatever was
    removed. Assumes a has at least two elements.

    >>> stuff = ['Cool!', 37, False]
    >>> pop_first(stuff)
    >>> stuff
    [False, 37]
    """
    a[0] = a.pop()


def random_list(n):
    """Returns list of length n with random integers between 0 and n."""
    a = []
    for _ in range(n):
        a.append(randrange(n))
    return a


def build_streak(m, n):
    """Returns list of n consecutive integers starting at m.

    >>> build_streak(5, 7)
    [5, 6, 7, 8, 9, 10, 11]
    """
    a = []
    for i in range(m, m + n):
        a.append(i)
    return a


def swap(a, i, j):
    """
    Exchanges contents of positions i and j in list a. Assumes i and j are
    valid indexes (0 <= i, j < len(a)).

    >>> stuff = ['Cool!', 37, False]
    >>> swap(stuff, 0, 2)
    >>> stuff
    [False, 37, 'Cool!']
    """
    t = a[i]
    a[i] = a[j]
    a[j] = t


def insert_at(a, i, x):
    """Inserts x at position i in list a. Assumes 0 <= i <= len(x).

    >>> stuff = []
    >>> insert_at(stuff, 0, 'alpha')
    >>> stuff
    ['alpha']
    >>> insert_at(stuff, 1, 'beta')
    >>> stuff
    ['alpha', 'beta']
    >>> insert_at(stuff, 1, 'gamma')
    >>> stuff
    ['alpha', 'gamma', 'beta']
    >>> insert_at(stuff, 1, 'delta')
    >>> stuff
    ['alpha', 'delta', 'gamma', 'beta']
    """
    j = len(a)
    a.append(None)
    while j > i:
        a[j] = a[j-1]
        j -= 1
    a[i] = x


def bubble_sort(a):
    """Sorts elements of a in increasing order using simplified bubble sort.

    >>> ns = [17, 3, 11, 19, 5, 7, 13]
    >>> bubble_sort(ns)
    >>> ns
    [3, 5, 7, 11, 13, 17, 19]
    """
    for _ in a:
        for i in range(1, len(a)):
            if a[i-1] > a[i]:
                swap(a, i-1, i)


# this is a pure function - no side effects - so it works for tuples or lists
# assumes 0 <= start < len(a)
def find_min_pos(a, start):
    mp = start
    for i in range(start + 1, len(a)):
        if a[i] < a[mp]:
            mp = i
    return mp


def selection_sort(a):
    """Sorts elements of a in increasing order using selection sort.

    >>> ns = [17, 3, 11, 19, 5, 7, 13]
    >>> selection_sort(ns)
    >>> ns
    [3, 5, 7, 11, 13, 17, 19]
    """
    for i in range(len(a) - 1):
        j = find_min_pos(a, i)
        swap(a, i, j)
